==> brushable_widget-0.1.2-py2.py3-none-any.whl: brushable_widget/static/widget.js <==
import { brush } from "https://esm.sh/d3-brush@3";
import { select, selectAll } from "https://esm.sh/d3-selection@3";
import RBush from 'https://cdn.jsdelivr.net/npm/rbush/+esm';

class MyRBush extends RBush {
    toBBox(node) { return { id: node.id, minX: node.cx, minY: node.cy, maxX: node.cx, maxY: node.cy }; }
    compareMinX(a, b) { return a.cx - b.cx; }
    compareMinY(a, b) { return a.cy - b.cy; }
}

let tree;
let svg;

const create_rtree = (data) => {
    tree = new MyRBush();
    tree.load(data);
    return tree;
}

function render({ model, el }) {
    let svg_string = model.get("svg");
    let input_svg;

    // Initialize SVG structure ONCE
    if (!svg) {
        let parser = new DOMParser();
        let svg_doc = parser.parseFromString(svg_string, "image/svg+xml");
        input_svg = svg_doc.documentElement;

        // Use D3 to create the SVG element within the widget
        svg = select(el).append("svg")
            .attr("width", input_svg.getAttribute("width"))
            .attr("height", input_svg.getAttribute("height"));

        // svg = svg.append("g"); // Add a group for the SVG content
        // svg = svg
        svg.html(svg_string); // Set the SVG content using the string.
        // svg.html(svg_string); // Set the SVG content using the string.

    } else {
        // Update SVG content if the string changes.
        let current_svg = svg.html();
        if(current_svg != svg_string) {
            svg.html(svg_string);
        }
    }

    // let svg = model.get("svg");
    // el.innerHTML = svg;

    // let parser = new DOMParser();
    // let svg_doc = parser.parseFromString(svg, "image/svg+xml");
    // let input_svg = svg_doc.documentElement;
    let svg_width = input_svg.getAttribute("width");
    let svg_height = input_svg.getAttribute("height");

    let brush_active = false;

    let brushable_items = svg.selectAll(".brushable");
    let brushable_items_data = Array.from(brushable_items).map(brushable_item => ({
        id: brushable_item.getAttribute("id"),
        cx: parseFloat(brushable_item.getAttribute("cx")),
        cy: parseFloat(brushable_item.getAttribute("cy"))
    }));
    tree = create_rtree(brushable_items_data);

    let overlay = select(el).select("svg")
        .append("svg")
        .attr("id", "overlay")
        .attr("class", "notebook")
        .style("position", "absolute")
        .style("top", 0)
        .style("left", 0)
        .style("width", svg_width)
        .style("height", svg_height)
        .style("pointer-events", "none");

    let activate_brush = () => {
        brush_active = true;
        el.style.pointerEvents = "none";
        overlay.style.pointerEvents = "auto";

        if (!overlay.select("#brush_group").empty()) return;
        overlay.insert("g", ":first-child")
            .attr("id", "brush_group")
            .attr("class", "brush")
            .call(my_brush);
    }

    let disactivate_brush = () => {
        brush_active = false;
        el.style.pointerEvents = "auto";
        overlay.style.pointerEvents = "none";

        overlay.select("#brush_group").remove();
    }

    let brushed = (event) => {
        if (event.selection) {
            brush_active = true;
            let [[x0_screen, y0_screen], [x1_screen, y1_screen]] = event.selection;
            let bbox = {
                minX: Math.min(x0_screen, x1_screen),
                minY: Math.min(y0_screen, y1_screen),
                maxX: Math.max(x0_screen, x1_screen),
                maxY: Math.max(y0_screen, y1_screen)
            };
            let selected_points = tree.search(bbox);
            let selected_ids = selected_points.map(node => node.id);
            model.set("selected_ids", selected_ids);
            model.save_changes();
        }
    };

    let my_brush = brush()
        .filter((event) => {
            // Prevent contextual menu on Ctrl-click (macOS)
            if (event.ctrlKey && event.button === 0) {
                event.preventDefault();
            }
            return (
                !event.button && // Ignore mouse buttons other than left-click
                (event.metaKey || event.ctrlKey || event.target.__data__.type !== "overlay")
            );
        })
        .extent([[0, 0], [svg_width, svg_height]])
        .on("start brush end", brushed);

    model.on("change:selected_ids", () => {
        let selected_ids = model.get("selected_ids");
        svg.selectAll('.brushable')
            .classed('brushed', function() {
                return selected_ids.includes(this.id);
        });
    })

    window.addEventListener("keydown", (e) => {
        if (e.metaKey && !brush_active) { activate_brush(); }
    });
    window.addEventListener("dblclick", (e) => {
        if (brush_active) { disactivate_brush(); }
    });

}
export default { render };

==> brushable_widget-0.1.4-py2.py3-none-any.whl: brushable_widget/static/widget.js <==
import { brush } from "https://esm.sh/d3-brush@3";
import { select, selectAll } from "https://esm.sh/d3-selection@3";
import RBush from 'https://cdn.jsdelivr.net/npm/rbush/+esm';

class MyRBush extends RBush {
    toBBox(node) { return { id: node.id, minX: node.x, minY: node.y, maxX: node.x, maxY: node.y }; }
    compareMinX(a, b) { return a.x - b.x; }
    compareMinY(a, b) { return a.y - b.y; }
}

let svg_string;
let tree;
let svg;
let svg_width;
let svg_height;
let parser;
let svg_doc;
let input_svg;
let brushable_items;
let brushable_items_data;

let brush_active = false;

const create_rtree = (data) => {
    tree = new MyRBush();
    tree.load(data);
    return tree;
}

function createIdentityMatrix() {
    const svg = document.createElementNS("http://www.w3.org/2000/svg", "svg");
    return svg.createSVGMatrix();
}

// Function to get the cumulative transformation matrix
function getCumulativeTransform(element) {
    let transform = createIdentityMatrix();
    while (element.parentNode && element.parentNode instanceof SVGElement) {
        element = element.parentNode;
        if (element.tagName === 'g') {
            transform = transform.multiply(element.getCTM());
        }
    }
    return transform;
}

const get_center = (element) => {
    let bbox = element.getBBox();

    let cumulativeTransform = getCumulativeTransform(element);

    let centerX = cumulativeTransform.a * (bbox.x + bbox.width / 2) + cumulativeTransform.c * (bbox.y + bbox.height / 2) + cumulativeTransform.e;
    let centerY = cumulativeTransform.b * (bbox.x + bbox.width / 2) + cumulativeTransform.d * (bbox.y + bbox.height / 2) + cumulativeTransform.f;

    return { x: centerX, y: centerY };
}

function initialize({ model }) {
    svg_string = model.get("svg");

    parser = new DOMParser();
    svg_doc = parser.parseFromString(svg_string, "image/svg+xml");
    input_svg = svg_doc.documentElement;

    svg_width = input_svg.getAttribute("width");
    svg_height = input_svg.getAttribute("height");

    brush_active = false;

}

function render({ model, el }) {
    // Things to do ONLY ONCE
    if (!svg) {
        parser = new DOMParser();
        svg_doc = parser.parseFromString(svg_string, "image/svg+xml");
        input_svg = svg_doc.documentElement;

        // Use D3 to create the SVG element within the widget
        svg = select(el).append("svg")
            .attr("width", input_svg.getAttribute("width"))
            .attr("height", input_svg.getAttribute("height"));

        svg.html(svg_string); // Set the SVG content using the string.

        brushable_items = svg.selectAll(".brushable");
        brushable_items_data = Array.from(brushable_items).map(brushable_item => ({
            id: brushable_item.getAttribute("id"),
            x: get_center(brushable_item).x,
            y: get_center(brushable_item).y
            // cx: parseFloat(brushable_item.getAttribute("cx")),
            // cy: parseFloat(brushable_item.getAttribute("cy"))
        }));
        
        tree = create_rtree(brushable_items_data);

    } else {
        // Update SVG content if the string changes.
        let current_svg = svg.html();
        if(current_svg != svg_string) {
            svg.html(svg_string);
        }
    }

    let overlay = select(el).select("svg")
        .append("svg")
        .attr("id", "overlay")
        .attr("class", "notebook")
        .style("position", "absolute")
        .style("top", 0)
        .style("left", 0)
        .style("width", svg_width)
        .style("height", svg_height)
        .style("pointer-events", "none");

    let activate_brush = () => {
        brush_active = true;
        el.style.pointerEvents = "none";
        overlay.style.pointerEvents = "auto";

        if (!overlay.select("#brush_group").empty()) return;
        overlay.insert("g", ":first-child")
            .attr("id", "brush_group")
            .attr("class", "brush")
            .call(my_brush);
    }

    let disactivate_brush = () => {
        brush_active = false;
        el.style.pointerEvents = "auto";
        overlay.style.pointerEvents = "none";

        overlay.select("#brush_group").remove();
    }

    let brushed = (event) => {
        if (event.selection) {
            brush_active = true;
            let [[x0_screen, y0_screen], [x1_screen, y1_screen]] = event.selection;
            let bbox = {
                minX: Math.min(x0_screen, x1_screen),
                minY: Math.min(y0_screen, y1_screen),
                maxX: Math.max(x0_screen, x1_screen),
                maxY: Math.max(y0_screen, y1_screen)
            };
            let selected_points = tree.search(bbox);
            let selected_ids = selected_points.map(node => node.id);
            model.set("selected_ids", selected_ids);
            model.save_changes();
        }
    };

    let my_brush = brush()
        .filter((event) => {
            // Prevent contextual menu on Ctrl-click (macOS)
            if (event.ctrlKey && event.button === 0) {
                event.preventDefault();
            }
            return (
                !event.button && // Ignore mouse buttons other than left-click
                (event.metaKey || event.ctrlKey || event.target.__data__.type !== "overlay")
            );
        })
        .extent([[0, 0], [svg_width, svg_height]])
        .on("start brush end", brushed);

    model.on("change:selected_ids", () => {
        let selected_ids = model.get("selected_ids");
        svg.selectAll('.brushable')
            .classed('brushed', function() {
                return selected_ids.includes(this.id);
        });
    })

    window.addEventListener("keydown", (e) => {
        if (e.metaKey && !brush_active) { activate_brush(); }
    });
    window.addEventListener("dblclick", (e) => {
        if (brush_active) { disactivate_brush(); }
    });

}
export default { initialize, render };
